#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This module contains project version information.

.. currentmodule:: ghostbuster.version
.. moduleauthor:: shubham_shah <sshah@assetnote.io>
"""

__version__ = "0.0.3"  #: the working version
__release__ = "0.0.3"  #: the release version
